@@ -29,7 +29,7 @@
 #include <errno.h>
 
 #include "usrp_prims.h"
-#include "spi_defs.h"
+#include "usrp_spi_defs.h"
 
 char *prog_name;
 
