@@ -46,7 +46,7 @@ static struct rate_to_regval rate_table[] = {
 // usb interface.  The kernel ends up locking down this much memory.
 
 static const int FUSB_BUFFER_SIZE = 8 * (1L << 20);	// 8 MB
-static const int FUSB_BLOCK_SIZE = 16 * (1L << 10);	// 16KB is hard limit
+static const int FUSB_BLOCK_SIZE = fusb_sysconfig::max_block_size();
 static const int FUSB_NBLOCKS    = FUSB_BUFFER_SIZE / FUSB_BLOCK_SIZE;
 
 
@@ -412,6 +412,10 @@ usrp0_rx::get_rx_freq (int channel) const
   return d_rx_freq[channel];
 }
 
+int
+usrp0_rx::block_size () const { return d_ephandle->block_size(); }
+
+
 ////////////////////////////////////////////////////////////////
 //
 //			   usrp0_tx
@@ -595,6 +599,9 @@ usrp0_tx::get_tx_freq (int channel) const
   return d_tx_freq[channel];
 }
 
+int
+usrp0_tx::block_size () const { return d_ephandle->block_size(); }
+
 bool
 usrp0_tx::_set_interp_reg (unsigned int regval)
 {
