@@ -42,8 +42,9 @@ using namespace ad9862;
 // These set the buffer size used for each end point using the fast
 // usb interface.  The kernel ends up locking down this much memory.
 
-static const int FUSB_BUFFER_SIZE = 8 * (1L << 20);	// 8 MB
-static const int FUSB_BLOCK_SIZE = 16 * (1L << 10);	// 16KB is hard limit
+static const int FUSB_BUFFER_SIZE = 2 * (1L << 20);	// 2 MB (was 8 MB)
+//static const int FUSB_BUFFER_SIZE = 256 * (1L << 10);	// 256 kB
+static const int FUSB_BLOCK_SIZE = fusb_sysconfig::max_block_size();
 static const int FUSB_NBLOCKS    = FUSB_BUFFER_SIZE / FUSB_BLOCK_SIZE;
 
 
@@ -139,6 +140,7 @@ usrp_basic::usrp_basic (int which_board,
   }
 
   _write_fpga_reg (FR_MODE, 0);		// ensure we're in normal mode
+  _write_fpga_reg (FR_DEBUG_EN, 0);	// disable debug outputs
 }
 
 usrp_basic::~usrp_basic ()
@@ -148,7 +150,13 @@ usrp_basic::~usrp_basic ()
 }
 
 bool
-usrp_basic::initialize ()
+usrp_basic::start ()
+{
+  return true;		// nop
+}
+
+bool
+usrp_basic::stop ()
 {
   return true;		// nop
 }
@@ -227,29 +235,10 @@ usrp_basic::read_i2c (int i2c_addr, int len)
 bool
 usrp_basic::set_adc_offset (int which, int offset)
 {
-  int	r, v;
-
-  switch (which){
-  case 0:
-    r = FR_ADC_OFFSET_1_0;
-    v = (d_fpga_shadows[r] & 0xffff0000) | (offset & 0xffff);
-    break;
-  case 1:
-    r = FR_ADC_OFFSET_1_0;
-    v = (d_fpga_shadows[r] & 0x0000ffff) | ((offset & 0xffff) << 16);
-    break;
-  case 2:
-    r = FR_ADC_OFFSET_3_2;
-    v = (d_fpga_shadows[r] & 0xffff0000) | (offset & 0xffff);
-    break;
-  case 3:
-    r = FR_ADC_OFFSET_3_2;
-    v = (d_fpga_shadows[r] & 0x0000ffff) | ((offset & 0xffff) << 16);
-    break;
-  default:
+  if (which < 0 || which > 3)
     return false;
-  }
-  return _write_fpga_reg (r, v);
+
+  return _write_fpga_reg (FR_ADC_OFFSET_0 + which, offset);
 }
 
 bool
@@ -314,7 +303,7 @@ bool
 usrp_basic::_write_fpga_reg (int regno, int value)
 {
   if (d_verbose){
-    fprintf (stdout, "_write_fpga_reg (%3d, 0x%08x)\n", regno, value);
+    fprintf (stdout, "_write_fpga_reg(%3d, 0x%08x)\n", regno, value);
     fflush (stdout);
   }
 
@@ -342,6 +331,12 @@ usrp_basic::_read_fpga_reg (int regno)
 bool
 usrp_basic::_write_9862 (int which_codec, int regno, unsigned char value)
 {
+  if (0 && d_verbose){
+    // FIXME really want to enable logging in usrp_prims:usrp_9862_write
+    fprintf(stdout, "_write_9862(codec = %d, regno = %2d, val = 0x%02x)\n", which_codec, regno, value);
+    fflush(stdout);
+  }
+
   return usrp_9862_write (d_udh, which_codec, regno, value);
 }
 
@@ -361,20 +356,34 @@ usrp_basic::_read_9862 (int which_codec, int regno) const
   return value;
 }
 
+bool
+usrp_basic::_write_spi (int optional_header, int enables, int format, std::string buf)
+{
+  return usrp_spi_write (d_udh, optional_header, enables, format,
+			 buf.data(), buf.size());
+}
+
+std::string
+usrp_basic::_read_spi (int optional_header, int enables, int format, int len)
+{
+  if (len <= 0)
+    return "";
+  
+  char buf[len];
+
+  if (!usrp_spi_read (d_udh, optional_header, enables, format, buf, len))
+    return "";
+
+  return std::string (buf, len);
+}
+
+
 bool
 usrp_basic::_set_led (int which, bool on)
 {
   return usrp_set_led (d_udh, which, on);
 }
 
-#if 0
-bool
-usrp_basic::_set_fpga_reset (bool on)
-{
-  return usrp_set_fpga_reset (d_udh, on);
-}
-#endif
-
 ////////////////////////////////////////////////////////////////
 //
 //			   usrp_basic_rx
@@ -392,7 +401,7 @@ static unsigned char rx_init_regs[] = {
 };
 
 
-usrp_basic_rx::usrp_basic_rx (int which_board)
+usrp_basic_rx::usrp_basic_rx (int which_board, int fusb_block_size, int fusb_nblocks)
   : usrp_basic (which_board, open_rx_interface),
     d_devhandle (0), d_ephandle (0),
     d_bytes_seen (0), d_first_read (true),
@@ -417,11 +426,34 @@ usrp_basic_rx::usrp_basic_rx (int which_board)
 
   set_fpga_rx_sample_rate_divisor (2);	// usually correct
 
-  probe_rx_slots (true);
+  set_dc_offset_cl_enable(0xf, 0xf);	// enable DC offset removal control loops
+
+  probe_rx_slots (false);
+
+  // check fusb buffering parameters
+
+  if (fusb_block_size < 0 || fusb_block_size > FUSB_BLOCK_SIZE)
+    throw std::out_of_range ("usrp_basic_rx: invalid fusb_block_size");
+
+  if (fusb_nblocks < 0)
+    throw std::out_of_range ("usrp_basic_rx: invalid fusb_nblocks");
+  
+  if (fusb_block_size == 0)
+    fusb_block_size = FUSB_BLOCK_SIZE;
+
+  if (fusb_nblocks == 0)
+    fusb_nblocks = std::max (1, FUSB_BUFFER_SIZE / fusb_block_size);
 
   d_devhandle = fusb_sysconfig::make_devhandle (d_udh);
   d_ephandle = d_devhandle->make_ephandle (USRP_RX_ENDPOINT, true,
-					   FUSB_BLOCK_SIZE, FUSB_NBLOCKS);
+					   fusb_block_size, fusb_nblocks);
+
+  _write_fpga_reg(FR_ATR_MASK_1, 0);	// zero Rx side Auto Transmit/Receive regs
+  _write_fpga_reg(FR_ATR_TXVAL_1, 0);
+  _write_fpga_reg(FR_ATR_RXVAL_1, 0);
+  _write_fpga_reg(FR_ATR_MASK_3, 0);
+  _write_fpga_reg(FR_ATR_TXVAL_3, 0);
+  _write_fpga_reg(FR_ATR_RXVAL_3, 0);
 }
 
 static unsigned char rx_fini_regs[] = {
@@ -446,9 +478,9 @@ usrp_basic_rx::~usrp_basic_rx ()
 
 
 bool
-usrp_basic_rx::initialize ()
+usrp_basic_rx::start ()
 {
-  if (!usrp_basic::initialize ())	// invoke parent's method
+  if (!usrp_basic::start ())	// invoke parent's method
     return false;
 
   // fire off reads before asserting rx_enable
@@ -468,18 +500,31 @@ usrp_basic_rx::initialize ()
   return true;
 }
 
+bool
+usrp_basic_rx::stop ()
+{
+  bool ok = usrp_basic::stop();
+
+  if (!d_ephandle->stop()){
+    fprintf (stderr, "usrp_basic_rx: failed to stop end point streaming");
+    usb_strerror ();
+    ok = false;
+  }
+  if (!set_rx_enable(false)){
+    fprintf (stderr, "usrp_basic_rx: set_rx_enable(false) failed\n");
+    usb_strerror ();
+    ok = false;
+  }
+  return false;
+}
 
 usrp_basic_rx *
-usrp_basic_rx::make (int which_board)
+usrp_basic_rx::make (int which_board, int fusb_block_size, int fusb_nblocks)
 {
   usrp_basic_rx *u = 0;
   
   try {
-    u = new usrp_basic_rx (which_board);
-    if (!u->initialize ()){
-      fprintf (stderr, "usrp_basic_rx::make failed to initialize\n");
-      throw std::runtime_error ("usrp_basic_rx::make");
-    }
+    u = new usrp_basic_rx (which_board, fusb_block_size, fusb_nblocks);
     return u;
   }
   catch (...){
@@ -643,18 +688,21 @@ usrp_basic_rx::probe_rx_slots (bool verbose)
       set_adc_offset (2*i+0, eeprom.offset[0]);
       set_adc_offset (2*i+1, eeprom.offset[1]);
       _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | eeprom.oe);
+      _write_fpga_reg (slot_id_to_io_reg(slot_id), (0xffff << 16) | 0x0000);
       break;
       
     case UDBE_NO_EEPROM:
       d_dbid[i] = -1;
       msg = "<none>";
       _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | 0x0000);
+      _write_fpga_reg (slot_id_to_io_reg(slot_id), (0xffff << 16) | 0x0000);
       break;
       
     case UDBE_INVALID_EEPROM:
       d_dbid[i] = -2;
       msg = "Invalid EEPROM contents";
       _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | 0x0000);
+      _write_fpga_reg (slot_id_to_io_reg(slot_id), (0xffff << 16) | 0x0000);
       break;
       
     case UDBE_BAD_SLOT:
@@ -734,6 +782,16 @@ usrp_basic_rx::read_aux_adc (int which_dboard, int which_adc)
   return usrp_basic::read_aux_adc (dboard_to_slot (which_dboard), which_adc);
 }
 
+int
+usrp_basic_rx::block_size () const { return d_ephandle->block_size(); }
+
+bool
+usrp_basic_rx::set_dc_offset_cl_enable(int bits, int mask)
+{
+  return _write_fpga_reg(FR_DC_OFFSET_CL_EN, 
+			 (d_fpga_shadows[FR_DC_OFFSET_CL_EN] & ~mask) | (bits & mask));
+}
+
 ////////////////////////////////////////////////////////////////
 //
 //			   usrp_basic_tx
@@ -773,7 +831,7 @@ static unsigned char tx_init_regs[] = {
   REG_TX_NCO_FTW_23_16,	0
 };
 
-usrp_basic_tx::usrp_basic_tx (int which_board)
+usrp_basic_tx::usrp_basic_tx (int which_board, int fusb_block_size, int fusb_nblocks)
   : usrp_basic (which_board, open_tx_interface),
     d_devhandle (0), d_ephandle (0),
     d_bytes_seen (0), d_first_write (true),
@@ -798,11 +856,33 @@ usrp_basic_tx::usrp_basic_tx (int which_board)
 
   set_fpga_tx_sample_rate_divisor (4);	// we're using interp x4
 
-  probe_tx_slots (true);
+  probe_tx_slots (false);
+
+  // check fusb buffering parameters
+
+  if (fusb_block_size < 0 || fusb_block_size > FUSB_BLOCK_SIZE)
+    throw std::out_of_range ("usrp_basic_rx: invalid fusb_block_size");
+
+  if (fusb_nblocks < 0)
+    throw std::out_of_range ("usrp_basic_rx: invalid fusb_nblocks");
+  
+  if (fusb_block_size == 0)
+    fusb_block_size = FUSB_BLOCK_SIZE;
+
+  if (fusb_nblocks == 0)
+    fusb_nblocks = std::max (1, FUSB_BUFFER_SIZE / fusb_block_size);
 
   d_devhandle = fusb_sysconfig::make_devhandle (d_udh);
   d_ephandle = d_devhandle->make_ephandle (USRP_TX_ENDPOINT, false,
-					   FUSB_BLOCK_SIZE, FUSB_NBLOCKS);
+					   fusb_block_size, fusb_nblocks);
+
+  _write_fpga_reg(FR_ATR_CTL, 0);		// zero Tx side Auto Transmit/Receive regs
+  _write_fpga_reg(FR_ATR_MASK_0, 0);	
+  _write_fpga_reg(FR_ATR_TXVAL_0, 0);
+  _write_fpga_reg(FR_ATR_RXVAL_0, 0);
+  _write_fpga_reg(FR_ATR_MASK_2, 0);
+  _write_fpga_reg(FR_ATR_TXVAL_2, 0);
+  _write_fpga_reg(FR_ATR_RXVAL_2, 0);
 }
 
 
@@ -822,12 +902,14 @@ usrp_basic_tx::~usrp_basic_tx ()
   if (!usrp_9862_write_many_all (d_udh, tx_fini_regs, sizeof (tx_fini_regs))){
     fprintf (stderr, "usrp_basic_tx: failed to fini AD9862 TX regs\n");
   }
+
+  _write_fpga_reg(FR_ATR_CTL, 0);	// disable Auto T/R
 }
 
 bool
-usrp_basic_tx::initialize ()
+usrp_basic_tx::start ()
 {
-  if (!usrp_basic::initialize ())
+  if (!usrp_basic::start ())
     return false;
 
   if (!set_tx_enable (true)){
@@ -845,17 +927,31 @@ usrp_basic_tx::initialize ()
   return true;
 }
 
+bool
+usrp_basic_tx::stop ()
+{
+  bool ok = usrp_basic::stop ();
+
+  if (!set_tx_enable (false)){
+    fprintf (stderr, "usrp_basic_tx: set_tx_enable(false) failed\n");
+    usb_strerror ();
+    ok = false;
+  }
+  if (!d_ephandle->stop ()){
+    fprintf (stderr, "usrp_basic_tx: failed to stop end point streaming");
+    usb_strerror ();
+    ok = false;
+  }
+  return ok;
+}
+
 usrp_basic_tx *
-usrp_basic_tx::make (int which_board)
+usrp_basic_tx::make (int which_board, int fusb_block_size, int fusb_nblocks)
 {
   usrp_basic_tx *u = 0;
   
   try {
-    u = new usrp_basic_tx (which_board);
-    if (!u->initialize ()){
-      fprintf (stderr, "usrp_basic_tx::make failed to initialize\n");
-      throw std::runtime_error ("usrp_basic_tx::make");
-    }
+    u = new usrp_basic_tx (which_board, fusb_block_size, fusb_nblocks);
     return u;
   }
   catch (...){
@@ -1002,18 +1098,21 @@ usrp_basic_tx::probe_tx_slots (bool verbose)
       // FIXME, figure out interpretation of dc offset for TX d'boards
       // offset = (eeprom.offset[1] << 16) | (eeprom.offset[0] & 0xffff);
       _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | eeprom.oe);
+      _write_fpga_reg (slot_id_to_io_reg(slot_id), (0xffff << 16) | 0x0000);
       break;
       
     case UDBE_NO_EEPROM:
       d_dbid[i] = -1;
       msg = "<none>";
       _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | 0x0000);
+      _write_fpga_reg (slot_id_to_io_reg(slot_id), (0xffff << 16) | 0x0000);
       break;
       
     case UDBE_INVALID_EEPROM:
       d_dbid[i] = -2;
       msg = "Invalid EEPROM contents";
       _write_fpga_reg (slot_id_to_oe_reg(slot_id), (0xffff << 16) | 0x0000);
+      _write_fpga_reg (slot_id_to_io_reg(slot_id), (0xffff << 16) | 0x0000);
       break;
       
     case UDBE_BAD_SLOT:
@@ -1093,3 +1192,6 @@ usrp_basic_tx::read_aux_adc (int which_dboard, int which_adc)
   return usrp_basic::read_aux_adc (dboard_to_slot (which_dboard), which_adc);
 }
 
+int
+usrp_basic_tx::block_size () const { return d_ephandle->block_size(); }
+
