@@ -69,11 +69,6 @@ protected:
   usrp_basic (int which_board,
 	      struct usb_dev_handle *open_interface (struct usb_device *dev));
 
-  /*!
-   * \brief called after construction in base class to derived class order
-   */
-  bool initialize ();
-
   /*!
    * \brief advise usrp_basic of usb data rate (bytes/sec)
    *
@@ -122,7 +117,7 @@ public:
   /*!
    * \brief return frequency of master oscillator on USRP
    */
-  long  fpga_master_clock_freq () const { return 128000000; }
+  long  fpga_master_clock_freq () const { return 64000000; }
 
   /*!
    * \returns usb data rate in bytes/sec
@@ -166,7 +161,7 @@ public:
    * \param i2c_addr		I2C bus address (7-bits)
    * \param len			number of bytes to read
    * \returns the data read if successful, else a zero length string.
-   * Reads are limited to a maximum of of 64 bytes.
+   * Reads are limited to a maximum of 64 bytes.
    */
   std::string read_i2c (int i2c_addr, int len);
 
@@ -194,6 +189,7 @@ public:
    */
   bool set_adc_buffer_bypass (int which, bool bypass);
 
+
   // ----------------------------------------------------------------
   // Low level implementation routines.
   // You probably shouldn't be using these...
@@ -249,6 +245,50 @@ public:
    * \returns register value if successful, else READ_FAILED
    */
   int  _read_9862 (int which_codec, int regno) const;
+
+  /*!
+   * \brief Write data to SPI bus peripheral.
+   *
+   * \param optional_header	0,1 or 2 bytes to write before buf.
+   * \param enables		bitmask of peripherals to write. See usrp_spi_defs.h
+   * \param format		transaction format.  See usrp_spi_defs.h SPI_FMT_*
+   * \param buf			the data to write
+   * \returns true iff successful
+   * Writes are limited to a maximum of 64 bytes.
+   *
+   * If \p format specifies that optional_header bytes are present, they are
+   * written to the peripheral immediately prior to writing \p buf.
+   */
+  bool _write_spi (int optional_header, int enables, int format, std::string buf);
+
+  /*
+   * \brief Read data from SPI bus peripheral.
+   *
+   * \param optional_header	0,1 or 2 bytes to write before buf.
+   * \param enables		bitmask of peripheral to read. See usrp_spi_defs.h
+   * \param format		transaction format.  See usrp_spi_defs.h SPI_FMT_*
+   * \param len			number of bytes to read.  Must be in [0,64].
+   * \returns the data read if sucessful, else a zero length string.
+   *
+   * Reads are limited to a maximum of 64 bytes.
+   *
+   * If \p format specifies that optional_header bytes are present, they
+   * are written to the peripheral first.  Then \p len bytes are read from
+   * the peripheral and returned.
+   */
+  std::string _read_spi (int optional_header, int enables, int format, int len);
+
+  /*!
+   * \brief Start data transfers.
+   * Called in base class to derived class order.
+   */
+  bool start ();
+
+  /*!
+   * \brief Stop data transfers.
+   * Called in base class to derived class order.
+   */
+  bool stop ();
 };
 
 /*!
@@ -266,10 +306,15 @@ private:
 protected:
   int			 d_dbid[2];		// Rx daughterboard ID's
 
-  usrp_basic_rx (int which_board);		// throws if trouble
-
-  // called after construction in base class to derived class order
-  bool initialize ();
+  /*!
+   * \param which_board	     Which USRP board on usb (not particularly useful; use 0)
+   * \param fusb_block_size  fast usb xfer block size.  Must be a multiple of 512. 
+   *                         Use zero for a reasonable default.
+   * \param fusb_nblocks     number of fast usb URBs to allocate.  Use zero for a reasonable default. 
+   */
+  usrp_basic_rx (int which_board,
+		 int fusb_block_size=0,
+		 int fusb_nblocks=0);  // throws if trouble
 
   bool set_rx_enable (bool on);
   bool rx_enable () const { return d_rx_enable; }
@@ -285,9 +330,15 @@ public:
 
   /*!
    * \brief invokes constructor, returns instance or 0 if trouble
+   *
+   * \param which_board	     Which USRP board on usb (not particularly useful; use 0)
+   * \param fusb_block_size  fast usb xfer block size.  Must be a multiple of 512. 
+   *                         Use zero for a reasonable default.
+   * \param fusb_nblocks     number of fast usb URBs to allocate.  Use zero for a reasonable default. 
    */
-  static usrp_basic_rx *make (int which_board);
-
+  static usrp_basic_rx *make (int which_board,
+			      int fusb_block_size=0,
+			      int fusb_nblocks=0);
 
   // MANIPULATORS
 
@@ -315,7 +366,9 @@ public:
   // ACCESSORS
 
   //! sampling rate of A/D converter
-  virtual long adc_freq () const { return fpga_master_clock_freq () / 2; } // 64M
+  virtual long converter_rate() const { return fpga_master_clock_freq(); } // 64M
+  long adc_rate() const { return converter_rate(); }
+  long adc_freq() const { return converter_rate(); }   //!< deprecated method name
 
   /*!
    * \brief Return daughterboard ID for given Rx daughterboard slot [0,1].
@@ -364,7 +417,7 @@ public:
   /*!
    * \brief Return hardware step size of PGA (linear in dB).
    */
-  double pga_db_per_step () const { return 1.0; }
+  double pga_db_per_step () const { return 20.0 / 20; }
 
   /*!
    * \brief Write direction register (output enables) for pins that go to daughterboard.
@@ -438,6 +491,37 @@ public:
    * \returns value in the range [0,4095] if successful, else READ_FAILED.
    */
   int read_aux_adc (int which_dboard, int which_adc);
+
+  /*!
+   * \brief returns current fusb block size
+   */
+  int block_size() const;
+
+  /*!
+   * \brief Enable/disable automatic DC offset removal control loop in FPGA
+   *
+   * \param bits  which control loops to enable
+   * \param mask  which \p bits to pay attention to
+   *
+   * If the corresponding bit is set, enable the automatic DC
+   * offset correction control loop.
+   *
+   * <pre>
+   * The 4 low bits are significant:
+   *
+   *   ADC0 = (1 << 0)
+   *   ADC1 = (1 << 1)
+   *   ADC2 = (1 << 2)
+   *   ADC3 = (1 << 3)
+   * </pre>
+   *
+   * By default the control loop is enabled on all ADC's.
+   */
+  bool set_dc_offset_cl_enable(int bits, int mask);
+
+  // called in base class to derived class order
+  bool start ();
+  bool stop ();
 };
 
 /*!
@@ -455,10 +539,15 @@ private:
  protected:
   int			 d_dbid[2];		// Tx daughterboard ID's
 
-  usrp_basic_tx (int which_board);		// throws if trouble
-
-  // called after construction in base class to derived class order
-  bool initialize ();
+  /*!
+   * \param which_board	     Which USRP board on usb (not particularly useful; use 0)
+   * \param fusb_block_size  fast usb xfer block size.  Must be a multiple of 512.
+   *                         Use zero for a reasonable default.
+   * \param fusb_nblocks     number of fast usb URBs to allocate.  Use zero for a reasonable default.
+   */
+  usrp_basic_tx (int which_board,
+		 int fusb_block_size=0,
+		 int fusb_nblocks=0);		// throws if trouble
 
   bool set_tx_enable (bool on);
   bool tx_enable () const { return d_tx_enable; }
@@ -475,8 +564,13 @@ public:
 
   /*!
    * \brief invokes constructor, returns instance or 0 if trouble
+   *
+   * \param which_board	     Which USRP board on usb (not particularly useful; use 0)
+   * \param fusb_block_size  fast usb xfer block size.  Must be a multiple of 512. 
+   *                         Use zero for a reasonable default.
+   * \param fusb_nblocks     number of fast usb URBs to allocate.  Use zero for a reasonable default. 
    */
-  static usrp_basic_tx *make (int which_board);
+  static usrp_basic_tx *make (int which_board, int fusb_block_size=0, int fusb_nblocks=0);
 
 
   // MANIPULATORS
@@ -484,7 +578,7 @@ public:
   /*!
    * \brief tell the fpga the rate tx samples are going to the D/A's
    *
-   * div = fpga_master_clock_freq () / sample_rate
+   * div = fpga_master_clock_freq () * 2
    *
    * sample_rate is determined by a myriad of registers
    * in the 9862.  That's why you have to tell us, so
@@ -512,7 +606,9 @@ public:
   // ACCESSORS
 
   //! sampling rate of D/A converter
-  virtual long dac_freq () const { return fpga_master_clock_freq (); } // 128M
+  virtual long converter_rate() const { return fpga_master_clock_freq () * 2; } // 128M
+  long dac_rate() const { return converter_rate(); }
+  long dac_freq() const { return converter_rate(); }	//!< deprecated method name
 
   /*!
    * \brief Return daughterboard ID for given Tx daughterboard slot [0,1].
@@ -561,7 +657,7 @@ public:
   /*!
    * \brief Return hardware step size of PGA (linear in dB).
    */
-  double pga_db_per_step () const { return 0.1; }
+  double pga_db_per_step () const { return 20.0/255; }
 
   /*!
    * \brief Write direction register (output enables) for pins that go to daughterboard.
@@ -635,6 +731,15 @@ public:
    * \returns value in the range [0,4095] if successful, else READ_FAILED.
    */
   int read_aux_adc (int which_dboard, int which_adc);
+
+  /*!
+   * \brief returns current fusb block size
+   */
+  int block_size() const;
+
+  // called in base class to derived class order
+  bool start ();
+  bool stop ();
 };
 
 #endif
